using System;
using Microsoft.VisualStudio.Package;
using Irony.Parsing;

namespace LuaLanguage
{
    public class LineScanner : IScanner
    {
        private Parser parser;
        private ParsingContext context;

        public LineScanner(Grammar grammar)
        {
            this.parser = new Parser(grammar);
            this.context = parser.Context;
            this.context.Mode = ParseMode.VsLineScan;
        }

        public bool ScanTokenAndProvideInfoAboutIt(TokenInfo tokenInfo, ref int state)
        {
            // Reads each token in a source line and performs syntax coloring.  It will continue to
            // be called for the source until false is returned.
            Token token = parser.Scanner.VsReadToken(ref state);

            // !EOL and !EOF
            if (token != null && token.Terminal != Grammar.CurrentGrammar.Eof && token.Category != TokenCategory.Error)
            {
                tokenInfo.StartIndex = token.Location.Position;
                tokenInfo.EndIndex = tokenInfo.StartIndex + token.Length - 1;

                TokenEditorInfo info = token.EditorInfo;

                tokenInfo.Color = (Microsoft.VisualStudio.Package.TokenColor)info.Color;
                tokenInfo.Type = (Microsoft.VisualStudio.Package.TokenType)info.Type;
                tokenInfo.Trigger =
                    (Microsoft.VisualStudio.Package.TokenTriggers)info.Triggers;
                
                return true;
            }

            return false;
        }

        public void SetSource(string source, int offset)
        {
            // Stores line of source to be used by ScanTokenAndProvideInfoAboutIt.
            parser.Scanner.VsSetSource(source, offset);
        }
    }
}
